@@ -21,7 +21,6 @@ import google.cloud.storage as gcs
 import requests
 from jsonschema import (
     FormatChecker,
-    ValidationError,
     validate,
 )
 from more_itertools import (
@@ -33,6 +32,19 @@ JSON = MutableMapping[str, T]
 
 log = logging.getLogger(__name__)
 
+staging_area_properties_schema = {
+    '$schema': 'https://json-schema.org/draft/2019-09/schema',
+    'properties': {
+        'is_delta': {
+            'type': 'boolean'
+        }
+    },
+    'required': [
+        'is_delta'
+    ],
+    'additionalProperties': False
+}
+
 
 class StagingAreaValidator:
 
@@ -63,6 +75,9 @@ class StagingAreaValidator:
         self.ignore_dangling_inputs = ignore_dangling_inputs
 
         self.gcs = gcs.Client()
+
+        # A boolean to tell us if this is a delta or non-delta staging area
+        self.is_delta = None
         # A mapping of data file name to metadata id
         self.names_to_id: MutableMapping[str, str] = {}
         # The status of each metadata file checked
@@ -97,11 +112,26 @@ class StagingAreaValidator:
 
     def _run(self):
         self.file_errors.clear()
-        self.validate_files('links')
-        self.validate_files('metadata')
-        self.validate_files('descriptors')
-        self.validate_files('data')
-        self.check_results()
+        self.validate_staging_area_properties()
+        if not self.file_errors:
+            self.validate_files('links')
+            self.validate_files('metadata')
+            self.validate_files('descriptors')
+            self.validate_files('data')
+            self.check_results()
+
+    def validate_staging_area_properties(self) -> None:
+        """
+        Verify and parse the staging area properties file.
+        """
+        print('Checking staging area properties')
+        properties_file_path = self.sa_path + 'staging_area.json'
+        blob = self.bucket.get_blob(properties_file_path)
+        assert isinstance(blob, gcs.Blob), properties_file_path
+        file_json = self.download_blob_as_json(blob)
+        self.validate_file_json(file_json, blob.name, staging_area_properties_schema)
+        self.is_delta = file_json['is_delta']
+        assert isinstance(self.is_delta, bool), file_json
 
     def validate_files(self, path: str) -> None:
         print(f'Checking files in {self.sa_path}{path}')
@@ -168,10 +198,11 @@ class StagingAreaValidator:
             file = self.metadata_files[entity_id]
         except KeyError:
             self.metadata_files[entity_id] = {
-                'name': None,
+                'name': set(),
                 'entity_id': entity_id,
                 'entity_type': entity_type,
-                'version': None,
+                'metadata_versions': set(),
+                'descriptor_versions': set(),
                 'project': {project_uuid},
                 'category': {category},
                 'found_metadata': False,
@@ -186,18 +217,17 @@ class StagingAreaValidator:
         metadata_type, metadata_file = blob.name.split('/')[-2:]
         assert metadata_file.count('_') == 1
         assert metadata_file.endswith('.json')
-        metadata_id, metadata_version = metadata_file.split('_')
+        metadata_id, metadata_version = metadata_file[:-5].split('_')
         file_json = self.download_blob_as_json(blob)
         self.validate_file_json(file_json, blob.name)
         if provenance := file_json.get('provenance'):
             assert metadata_id == provenance['document_id']
         if metadata_file := self.metadata_files.get(metadata_id):
-            metadata_file['name'] = blob.name
-            metadata_file['version'] = metadata_version
+            metadata_file['name'].add(blob.name)
+            metadata_file['metadata_versions'].add(metadata_version)
             metadata_file['found_metadata'] = True
             if metadata_type.endswith('_file'):
                 metadata_file['data_file_name'] = file_json['file_core']['file_name']
-                metadata_file['found_descriptor'] = False
                 metadata_file['found_data_file'] = False
             if metadata_type == 'supplementary_file' and file_json.get('provenance', {}).get('submitter_id'):
                 try:
@@ -235,19 +265,21 @@ class StagingAreaValidator:
 
     def validate_descriptors_file(self, blob: gcs.Blob) -> None:
         # Expected syntax: descriptors/{metadata_type}/{metadata_id}_{version}.json
-        metadata_type, metadata_file = blob.name.split('/')[-2:]
-        assert metadata_file.count('_') == 1
-        assert metadata_file.endswith('.json')
-        metadata_id, metadata_version = metadata_file.split('_')
+        metadata_type, descriptor_file = blob.name.split('/')[-2:]
+        assert descriptor_file.count('_') == 1
+        assert descriptor_file.endswith('.json')
+
+        metadata_id, descriptor_version = descriptor_file[:-5].split('_')
         file_json = self.download_blob_as_json(blob)
         self.validate_file_json(file_json, blob.name)
         file_name = file_json['file_name']
         self.names_to_id[file_name] = metadata_id
+
         if metadata_file := self.metadata_files.get(metadata_id):
-            metadata_file['found_descriptor'] = True
             metadata_file['crc32c'] = file_json['crc32c']
-            version = metadata_file['version']
-            assert version == metadata_version, f'{version} != {metadata_version}'
+            metadata_versions = metadata_file['metadata_versions']
+            assert descriptor_version in metadata_versions, f'Corresponding metadata version for descriptor version {descriptor_version} not found'
+            metadata_file['descriptor_versions'].add(descriptor_version)
         else:
             self.extra_files.append(blob.name)
 
@@ -264,10 +296,15 @@ class StagingAreaValidator:
         if metadata_file is None:
             self.extra_files.append(blob.name)
 
-    def validate_file_json(self, file_json: JSON, file_name: str) -> None:
+    def validate_file_json(self,
+                           file_json: JSON,
+                           file_name: str,
+                           schema: Optional[JSON] = None
+                           ) -> None:
         if self.validate_json:
+            print(f'Validating JSON of {file_name}')
             try:
-                self.validator.validate_json(file_json, file_name)
+                self.validator.validate_json(file_json, schema)
             except Exception as e:
                 log.error('File %s failed json validation.', file_name)
                 self.file_errors[file_name] = e
@@ -293,9 +330,13 @@ class StagingAreaValidator:
                                 one(metadata_file['project']))
                 else:
                     raise Exception('Did not find metadata file', metadata_file)
+            if self.is_delta and len(metadata_file['metadata_versions']) > 1:
+                raise Exception('Delta staging areas must not contain redundant '
+                                'versions of metadata.',
+                                metadata_file)
             if metadata_file['entity_type'].endswith('_file'):
-                if not metadata_file['found_descriptor']:
-                    raise Exception('Did not find descriptor file', metadata_file)
+                if not metadata_file['descriptor_versions'] == metadata_file['metadata_versions']:
+                    raise Exception('Did not find a matching descriptor file', metadata_file)
                 if not metadata_file['found_data_file']:
                     raise Exception('Did not find data file', metadata_file)
         try:
@@ -321,17 +362,17 @@ class StagingAreaValidator:
 class SchemaValidator:
 
     @classmethod
-    def validate_json(cls, file_json: JSON, file_name: str):
-        print(f'Validating JSON of {file_name}')
-        try:
-            schema = cls._download_schema(file_json['describedBy'])
-        except json.decoder.JSONDecodeError as e:
-            schema_url = file_json['describedBy']
-            raise Exception('Failed to parse schema JSON', file_name, schema_url) from e
-        try:
-            validate(file_json, schema, format_checker=FormatChecker())
-        except ValidationError as e:
-            raise ValidationError(f'File {file_name}') from e
+    def validate_json(cls,
+                      file_json: JSON,
+                      schema: Optional[JSON] = None
+                      ) -> None:
+        if schema is None:
+            try:
+                schema = cls._download_schema(file_json['describedBy'])
+            except json.decoder.JSONDecodeError as e:
+                schema_url = file_json['describedBy']
+                raise Exception('Failed to parse schema JSON', schema_url) from e
+        validate(file_json, schema, format_checker=FormatChecker())
 
     @classmethod
     @lru_cache
